# lab3_group_analysis.R
# Analysis of equiluminance motion threshold group data
# Lew Harvey
# 24 February 2014\
rm(list = ls())

COMPUTE_EFFECT_SIZE <- TRUE
if (COMPUTE_EFFECT_SIZE){
	if (!require(bootES)){
		print("package bootES is not found")
		print("You must install the package bootES from CRAN")
	}
}

# load the nlme library to use lme()
library(nlme)

# read in group data in wide format from Excel text file
fn <- file("lab3_group_data_2014_Spring.csv")
fn <- file.choose()
df.wide <- read.csv(fn, header=TRUE)
# compute the contrast column for bootstrap comparison of iso vs contrast
df.wide$con.30vs00 <- df.wide$jnd.00 - df.wide$jnd.30
# ******************************************************
# reshape the file into long format
# Note: reshape() is a bit of a pain to use but what it does
# is take data that are in a so-called wide format
# (one row per subject, separate columns per dependent variable)
# and put it into the long formate needed by many R analysis routines
# (multiple rows per subject, single column per dependent variable)
df.long <- reshape(df.wide,
	# old column names
	varying = list(c("mu.00", "mu.30"), c("jnd.00", "jnd.30"), c("aic.00", "aic.30")),
	idvar = "subject",
	times = c("00", "30"),
	# new column names
	v.names = c("mu", "jnd", "aic"),
	timevar = "elevation",
	direction = "long")
rownames(df.long) <- 1:dim(df.long)[1]
df.long$mu  <- as.numeric(df.long$mu)
df.long$jnd <- as.numeric(df.long$jnd)
df.long$aic <- as.numeric(df.long$aic)
df.long$elevation <- as.numeric(df.long$elevation)
# ******************************************************

# use shorter name for the data frame
df <- df.long

# ------------------------------------------------------
# Make Figure 1
# ------------------------------------------------------
plot1 <- function() {
	par(mfcol = c(2,1))
	plot1.strip()
	plot1.box()
	par(mfcol = c(1,1))
}
plot1.strip <- function() {
	# Make a strip chart of the motion jnd data
	stripchart(jnd ~ elevation, data = df, method = "jitter", jitter = 0.03,
 		vertical = TRUE,
 		xlim = c(0.5, 2.5),
 		ylim = c(0.0, 0.7),
		xlab = "Luminance Contrast (DKL elevation)",
		ylab = "Motion Coherence JND",
		main = "Luminance Contrast on Motion Perception")
}
plot1.box <- function() {
	# Make a box plot of the motion jnd data:
	boxplot(jnd ~ elevation, data = df,
		col = "yellow",
		vertical = FALSE,
 		xlim = c(0.5, 2.5),
		ylim = c(0.0, 0.7),
		xlab = "Luminance Contrast (DKL elevation)",
		ylab = "Motion Coherence JND",
		main = "Luminance Contrast on Motion Perception")
}


plot2 <- function() {
	par(mfcol = c(2,1))
	plot2.strip()
	plot2.box()
	par(mfcol = c(1,1))
}
plot2.strip <- function() {
	# Make a strip chart of the motion jnd data
	stripchart(aic ~ elevation, data = df, method = "jitter", jitter = 0.03,
 		vertical = TRUE,
 		xlim = c(0.5, 2.5),
 		ylim = c(0.0, 40.0),
		xlab = "Luminance Contrast (DKL elevation)",
		ylab = "Goodness-of-Fit (aic)",
		main = "Luminance Contrast on aic")
}
plot2.box <- function() {
	# Make a box plot of the motion jnd data:
	boxplot(aic ~ elevation, data = df,
		col = "yellow",
		vertical = FALSE,
 		xlim = c(0.5, 2.5),
		ylim = c(0.0, 40.0),
		xlab = "Luminance Contrast (DKL elevation)",
		ylab = "Goodness-of-Fit (aic)",
		main = "Luminance Contrast on aic")
}


# Testing the hypothesis that motion threshold (jnd) is independent of luminance contrast and gender
# Compute the mixed-effects (within and between factors ANOVA
lme.mod1 <- lme(jnd ~ elevation, random = ~ 1 | subject, data = df)

# print a summary of the analysis of variance
print(anova(lme.mod1))

# Print a table of means for the two luminance conditions
print("elevation means")
print(tapply(df$jnd, df$elevation, mean, na.rm=TRUE))



# compute effect size of contrst with bootstrap package bootES
if (COMPUTE_EFFECT_SIZE){
	df$elevation <- factor(df$elevation)
	bssamp <- 5000 # number of iterations
	df.00 <- subset(df, elevation == "0")
	df.30 <- subset(df, elevation == "30")
	
	print("computing the estimates and confidence intervals for the two contrasts separately")
	coherence.jnd.00 <- bootES(data = df.00, R = bssamp,
		data.col = "jnd", effect.type = "unstandardized", plot = FALSE)
	coherence.jnd.30 <- bootES(data = df.30, R = bssamp,
		data.col = "jnd", effect.type = "unstandardized", plot = FALSE)
		
	print("computing the Cohen's d estimates and confidence intervals for the two contrasts separately")
	coherence.jnd.00.d <- bootES(data = df.00, R = bssamp,
		data.col = "jnd", effect.type = "cohens.d", plot = FALSE)
	coherence.jnd.30.d <- bootES(data = df.30, R = bssamp,
		data.col = "jnd", effect.type = "cohens.d", plot = FALSE)
		
	print("computing the cohen's d and confidence intervals for the contrasts as within subjects effect")
	coherence.jnd.unstandardized <- bootES(data = df.wide, R = bssamp,
		data.col = "con.30vs00", effect.type = "unstandardized", plot = FALSE)
	coherence.jnd.cohens.d <- bootES(data = df, R = bssamp,
		data.col = "con.30vs00", effect.type = "cohens.d", plot = FALSE)

plot3 <- function(){
	par(mfcol = c(2,2))
	plot3.unstandardized.strip()
	plot3.cohens.d.strip()
	plot3.00vs30.unstandardized.strip()
	plot3.00vs30.cohens.d.strip()
	par(mfcol = c(1,1))
}
	plot3.unstandardized.strip <- function() {
		# make effect size plots from the bootstrap data
		df.tmp <- data.frame(contrast.level = c("0", "30"))
		df.tmp$bs.estimate <- c(coherence.jnd.00$t0, coherence.jnd.30$t0)
		df.tmp$ci.lower <- c(coherence.jnd.00$bounds[1], coherence.jnd.30$bounds[1])
		df.tmp$ci.upper <- c(coherence.jnd.00$bounds[2], coherence.jnd.30$bounds[2])
	
		stripchart(bs.estimate ~ contrast.level, data = df.tmp, vertical = TRUE,
			pch = 19, cex = 2,
			xlim = c(0.5, 2.5),
			ylim = c(min(0.0, min(df.tmp$ci.lower)), max(df.tmp$ci.upper)),
			xlab = "luminance contrast",
			ylab = "Motion Coherence JND",
			main = "Effect Size of Contrast Levels")
		abline(h = 0, col = "red")
			# add confidence intervals
		wd <- .05
		for (i in 1:(dim(df.tmp)[1])) {
			lines(c(i, i), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.upper"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.lower"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.upper"], df.tmp[i, "ci.upper"]), col = "blue")
		}
	}

	plot3.cohens.d.strip <- function() {
		# make effect size plots from the bootstrap data
		df.tmp <- data.frame(contrast.level = c("0", "30"))
		df.tmp$bs.estimate <- c(coherence.jnd.00.d$t0, coherence.jnd.30.d$t0)
		df.tmp$ci.lower <- c(coherence.jnd.00.d$bounds[1], coherence.jnd.30.d$bounds[1])
		df.tmp$ci.upper <- c(coherence.jnd.00.d$bounds[2], coherence.jnd.30.d$bounds[2])
	
		stripchart(bs.estimate ~ contrast.level, data = df.tmp, vertical = TRUE,
			pch = 19, cex = 2,
			xlim = c(0.5, 2.5),
			ylim = c(min(0.0, min(df.tmp$ci.lower)), max(df.tmp$ci.upper)),
			xlab = "luminance contrast",
			ylab = "Cohen's d Effect Size",
			main = "Standardized Effect Size of Contrast Levels")
		abline(h = 0, col = "red")
			# add confidence intervals
		wd <- .05
		for (i in 1:(dim(df.tmp)[1])) {
			lines(c(i, i), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.upper"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.lower"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.upper"], df.tmp[i, "ci.upper"]), col = "blue")
		}
	}


	plot3.00vs30.unstandardized.strip <- function() {
		# make effect size plots from the bootstrap data
		df.tmp <- data.frame(contrast.level = c("0 vs 30"))
		df.tmp$bs.estimate <- c(coherence.jnd.unstandardized$t0)
		df.tmp$ci.lower <- c(coherence.jnd.unstandardized$bounds[1])
		df.tmp$ci.upper <- c(coherence.jnd.unstandardized$bounds[2])
	
		stripchart(bs.estimate ~ contrast.level, data = df.tmp, vertical = TRUE,
			pch = 19, cex = 2,
			xlim = c(0.5, 1.5),
			ylim = c(min(0.0, min(df.tmp$ci.lower)), max(df.tmp$ci.upper)),
			xlab = "0 vs 30 luminance contrast effect",
			ylab = "Difference Motion Coherence JND",
			main = "Effect Size of Contrast Levels")
		abline(h = 0, col = "red")
			# add confidence intervals
		wd <- .05
		for (i in 1:(dim(df.tmp)[1])) {
			lines(c(i, i), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.upper"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.lower"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.upper"], df.tmp[i, "ci.upper"]), col = "blue")
		}
	}


	plot3.00vs30.cohens.d.strip <- function() {
		# make effect size plots from the bootstrap data
		df.tmp <- data.frame(contrast.level = c("0 vs 30"))
		df.tmp$bs.estimate <- c(coherence.jnd.cohens.d$t0)
		df.tmp$ci.lower <- c(coherence.jnd.cohens.d$bounds[1])
		df.tmp$ci.upper <- c(coherence.jnd.cohens.d$bounds[2])
	
		stripchart(bs.estimate ~ contrast.level, data = df.tmp, vertical = TRUE,
			pch = 19, cex = 2,
			xlim = c(0.5, 1.5),
			ylim = c(min(0.0, min(df.tmp$ci.lower)), max(df.tmp$ci.upper)),
			xlab = "0 vs 30 luminance contrast effect",
			ylab = "Cohen's d Effect Size",
			main = "Standardized Effect Size of Contrast Levels")
		abline(h = 0, col = "red")
			# add confidence intervals
		wd <- .05
		for (i in 1:(dim(df.tmp)[1])) {
			lines(c(i, i), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.upper"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.lower"], df.tmp[i, "ci.lower"]), col = "blue")
			lines(c(i-wd, i+wd), c(df.tmp[i, "ci.upper"], df.tmp[i, "ci.upper"]), col = "blue")
		}
	}

}

plot1()
par(ask = TRUE)
plot2()
plot3()
par(ask = FALSE)